﻿library CloseSplitEditor;

{$R 'mPlugin.res' 'mPlugin.rc'}


uses
  Winapi.Windows,
  System.SysUtils,
  System.Classes,
  System.IniFiles,
  mPlugin in 'mPlugin.pas',
  Native in 'Native.pas',
  mCommon in 'mCommon.pas';

resourcestring
  SName = '分割画面を閉じる';
  SVersion = '1.0.0';

const
  IDS_MENU_TEXT = 1;
  IDS_STATUS_MESSAGE = 2;
  IDI_ICON = 101;

{$IFDEF DEBUG}
{$R *.res}
{$ENDIF}

// EnumChildWindows のコールバック関数(TNotePad を探す)
function EnumNotePad(hWnd: HWND; lParam: TList) :BOOL; stdcall;
var
  ClassName :array[0..MAX_PATH] of Char;
begin
  Result := True;
  if not IsWindowVisible(hWnd) then
    Exit;

  GetClassName(hWnd, ClassName, Sizeof(ClassName));
  if ClassName = 'TNotePad' then
    lParam.Add(Pointer(hWnd));
end;

// -----------------------------------------------------------------------------
// OnCommand
// プラグインを実行した時に呼び出されます
// パラメータ
//   hwnd: ウィンドウのハンドル

procedure OnCommand(hwnd: HWND); stdcall;
var
  NotePadList: TList;
  NotePadRect: array[0..1] of TRect;
  CloseCommand: NativeUInt;
  SubEditor: Winapi.Windows.HWND;
  ScrollMargin, FontWidth: NativeInt;
  ScrollInfo: TScrollInfo;
  ScrollPos: TNativePoint;
  i: NativeInt;
  s: string;
begin
  NotePadList := TList.Create;
  try
    // TNotePad を探す
    EnumChildWindows(hwnd, @EnumNotePad, LPARAM(NotePadList));
    if NotePadList.Count < 2 then // 分割されていない
      Exit;

    for i := 0 to 1 do
      GetWindowRect(Winapi.Windows.HWND(NotePadList[i]), NotePadRect[i]);
    if NotePadRect[0].Left = NotePadRect[1].Left then // 左端が揃っているなら上下分割
    begin
      CloseCommand := MEID_WINDOW_SPLIT_HORZ;
      if NotePadRect[0].Top < NotePadRect[1].Top then
        SubEditor := Winapi.Windows.HWND(NotePadList[1])
      else
        SubEditor := Winapi.Windows.HWND(NotePadList[0]);
    end
    else // 左右分割
    begin
      CloseCommand := MEID_WINDOW_SPLIT_VERT;
      if NotePadRect[0].Left < NotePadRect[1].Left then
        SubEditor := Winapi.Windows.HWND(NotePadList[1])
      else
        SubEditor := Winapi.Windows.HWND(NotePadList[0]);
    end;
  finally
    NotePadList.Free;
  end;

  FontWidth := 7;
  ScrollMargin := 2;
  if GetIniFileName(s) then
  begin
    with TMemIniFile.Create(s, TEncoding.UTF8) do
    begin
      try
        FontWidth := (Abs(ReadInteger('Display', 'FontHeight', -13)) + 1) div 2;
        ScrollMargin := ReadInteger('General', 'ScrollMargin', 2);
      finally
        Free;
      end;
    end;
  end;

  ScrollInfo.cbSize := SizeOf(ScrollInfo);
  ScrollInfo.fMask := SIF_POS;
  GetScrollInfo(SubEditor, SB_HORZ, ScrollInfo);
  ScrollPos.X := ScrollInfo.nPos * FontWidth;
  GetScrollInfo(SubEditor, SB_VERT, ScrollInfo);
  ScrollPos.Y := ScrollInfo.nPos + ScrollMargin;
  Editor_SetScrollPos(hwnd, @ScrollPos);
  Editor_ExecCommand(hwnd, CloseCommand);
end;

// -----------------------------------------------------------------------------
// QueryStatus
//   プラグインが実行可能か、またはチェックされた状態かを調べます
// パラメータ
//   hwnd:      ウィンドウのハンドル
//   pbChecked: チェックの状態
// 戻り値
//   実行可能であればTrueを返します

function QueryStatus(hwnd: HWND; pbChecked: PBOOL): BOOL; stdcall;
begin
  pbChecked^ := False;
  Result := True;
end;

// -----------------------------------------------------------------------------
// GetMenuTextID
//   メニューに表示するテキストのリソース識別子を取得します
// 戻り値
//   リソース識別子

function GetMenuTextID: NativeInt; stdcall;
begin
  Result := IDS_MENU_TEXT;
end;

// -----------------------------------------------------------------------------
// GetStatusMessageID
//   ツールチップに表示するテキストのリソース識別子を取得します
// 戻り値
//   リソース識別子

function GetStatusMessageID: NativeInt; stdcall;
begin
  Result := IDS_STATUS_MESSAGE;
end;

// -----------------------------------------------------------------------------
// GetIconID
//   ツールバーに表示するアイコンのリソース識別子を取得します
// 戻り値
//   リソース識別子

function GetIconID: NativeInt; stdcall;
begin
  Result := IDI_ICON;
end;

// -----------------------------------------------------------------------------
// OnEvents
//   イベントが発生した時に呼び出されます
// パラメータ
//   hwnd:   ウィンドウのハンドル
//   nEvent: イベントの種類
//   lParam: メッセージ特有の追加情報
// 備考
//   EVENT_CREATE:             エディタを起動した時
//   EVENT_CLOSE:              エディタを終了した時
//   EVENT_CREATE_FRAME:       フレームを作成された時
//   EVENT_CLOSE_FRAME:        フレームが破棄された時
//   EVENT_SET_FOCUS:          フォーカスを取得した時
//   EVENT_KILL_FOCUS:         フォーカスを失った時
//   EVENT_FILE_OPENED:        ファイルを開いた時
//   EVENT_FILE_SAVED:         ファイルを保存した時
//   EVENT_MODIFIED:           更新の状態が変更された時
//   EVENT_CARET_MOVED:        カーソルが移動した時
//   EVENT_SCROLL:             スクロールされた時
//   EVENT_SEL_CHANGED:        選択範囲が変更された時
//   EVENT_CHANGED:            テキストが変更された時
//   EVENT_CHAR:               文字が入力された時
//   EVENT_MODE_CHANGED:       編集モードが変更された時
//   EVENT_DOC_SEL_CHANGED:    アクティブな文書が変更された時
//   EVENT_DOC_CLOSE:          文書を閉じた時
//   EVENT_TAB_MOVED:          タブが移動された時
//   EVENT_CUSTOM_BAR_CLOSING: カスタムバーを閉じようとした時
//   EVENT_CUSTOM_BAR_CLOSED:  カスタムバーを閉じた時
//   EVENT_TOOL_BAR_CLOSED:    ツールバーを閉じた時
//   EVENT_TOOL_BAR_SHOW:      ツールバーが表示された時
//   EVENT_IDLE:               アイドル時

procedure OnEvents(hwnd: HWND; nEvent: NativeInt; lParam: LPARAM); stdcall;
begin
  //
end;

// -----------------------------------------------------------------------------
// PluginProc
//   プラグインに送られるメッセージを処理します
// パラメータ
//   hwnd:   ウィンドウのハンドル
//   nMsg:   メッセージ
//   wParam: メッセージ特有の追加情報
//   lParam: メッセージ特有の追加情報
// 戻り値
//   メッセージにより異なります

function PluginProc(hwnd: HWND; nMsg: NativeInt; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
begin
  Result := 0;
  case nMsg of
    MP_GET_NAME:
      begin
        Result := Length(SName);
        if lParam <> 0 then
          lstrcpynW(PChar(lParam), PChar(SName), wParam);
      end;
    MP_GET_VERSION:
      begin
        Result := Length(SVersion);
        if lParam <> 0 then
          lstrcpynW(PChar(lParam), PChar(SVersion), wParam);
      end;
  end;
end;

exports
  OnCommand,
  QueryStatus,
  GetMenuTextID,
  GetStatusMessageID,
  GetIconID,
  OnEvents,
  PluginProc;

begin
{$IFDEF DEBUG}
  ReportMemoryLeaksOnShutdown := True;
{$ENDIF}

end.
